define([
    'jquery',
    'underscore',
    'backbone',
    'multi_upload_settings'
],
function($, _, Backbone, MultiUploadSettings) {
    'use strict';
    var UploadFile = Backbone.Model.extend({
        defaults: {
            index: null,
            label: '',
            thumb: false,
            width: null,
            height: null,
            imageWidth: null,
            imageHeight: null,
            align: 'none',
            popup: false,
            overwriteYes: false,
            content: '',
            resizedWidth: null,
            resizedHeight: null,
            editField: '',
            resize: 0,
            resizeSize: null,
            resizeWidth: null,
            imageDefaults: false,
            canSaveImageDefaults: false,
            sitePath: '',
            middlePath: '',
            destination: '',
            extraPath: '',
            customDestination: '',
            watermarkInsertion: false,
            encloseStartTag: '<p>',
            encloseEndTag: '</p>',
            file: null
        },
        initialize: function(attributes) {
            if(attributes.file) {
                this.attributes.label = attributes.file.name;
                this.loadContent();
            }
            this.attributes.resize = MultiUploadSettings.enable_resize;
            this.attributes.resizeSize = MultiUploadSettings.max_size;
            this.attributes.resizeWidth = MultiUploadSettings.max_width;
            if(!MultiUploadSettings.upload_compat) {
                if(MultiUploadSettings.allow_to_change_at_upload == 1) {
                    var dest = _.find(MultiUploadSettings.destination_loop, function(dest) {
                        return dest.selected;
                    });
                    if(!dest) {
                        if(MultiUploadSettings.destination_loop.length > 0) {
                            dest = MultiUploadSettings.destination_loop[0];
                        }
                        else {
                            dest = { path: '' };
                        }
                    }
                    this.attributes.destination = dest.path || '';
                }
                else {
                    this.attributes.destination = MultiUploadSettings.upload_destination_value || '';
                }
            }
            this.attributes.extraPath = MultiUploadSettings.extra_path || '';

            // image defaults settings
            this.attributes.canSaveImageDefaults = MultiUploadSettings.can_save_image_defaults;
            this.attributes.align = _.find(['none','left','center','right'], function(align) {
                return MultiUploadSettings['align_'+align] == 1;
            });
            this.attributes.popup = MultiUploadSettings.popup;
            this.attributes.thumb = MultiUploadSettings.make_thumb;

            this.attributes.encloseStartTag = MultiUploadSettings.enclose_start_tag || '<p>';
            this.attributes.encloseEndTag = MultiUploadSettings.enclose_end_tag || '</p>';
            this.lastResult = null;
            this.lastError = null;

            this.on('change:imageWidth', this.updateResizedSize);
            this.on('change:imageHeight', this.updateResizedSize);
            this.on('change:width', this.changeWidth);
        },
        _loadContent: function() {
            var file = this.attributes.file;
            var reader = new FileReader();
            var self = this;
            reader.onload = function(event) {
                if (window.Blob) {
                    var blob = new Blob([event.target.result]);
                    var blobURL = window.URL.createObjectURL(blob);
                    self.set({ content: blobURL });
                }
                else {
                    iuu.log('no Blob');
                }
            };
            reader.readAsArrayBuffer(file);
        },
        loadContent: function() {
            var file = this.attributes.file;
            var blobURL = window.URL.createObjectURL(file);
            var self = this;
            _.defer(function() {
                self.set('content', blobURL);
            });
        },
        cleanUp: function() {
            if(this.attributes.content) {
                window.URL.revokeObjectURL(this.attributes.content);
                // this.attributes.content = null;
                this.set({ content: null });
            }
        },
        changeWidth: function() {
            var ratio = this.attributes.width / this.attributes.imageWidth;
            var height = Math.round(this.attributes.imageHeight * ratio);
            this.set({ height: height });
        },
        updateResizedSize: function() {
            var w = this.attributes.imageWidth;
            var h = this.attributes.imageHeight;
            if(!w || !h) {
                return;
            }
            var rw = w;
            var rh = h;
            var ratio;
            if(MultiUploadSettings.thumb_width) {
                var tw = MultiUploadSettings.thumb_width;
                ratio = (w > tw) ? (tw / w) : 1;
                if(w > tw) {
                    rw = tw;
                    rh = Math.round(h * ratio);
                }
            }
            else {
                if(this.attributes.resize == 1) {
                    var maxSize = parseInt(this.attributes.resizeSize);
                    if(isNaN(maxSize)) {
                        iuu.log('invalid maxSize', this.attributes.resizeSize);
                        return;
                    }
                    var size = (w > h) ? w : h;
                    ratio = (size > maxSize) ? (maxSize / size) : 1;
                    if(w > maxSize){
                        rw = maxSize;
                        rh = Math.round(h * ratio);
                    }
                    else if(h > maxSize) {
                        rw = Math.round(w * ratio);
                        rh = maxSize;
                    }
                }
                else if(this.attributes.resize == 2) {
                    var maxWidth = this.attributes.resizeWidth;
                    if(isNaN(maxWidth)) {
                        iuu.log('invalid maxWidth', this.attributes.resizeWidth);
                        return;
                    }
                    ratio = (w > maxWidth) ? (maxWidth / w) : 1;
                    if(w > maxWidth) {
                        rw = maxWidth;
                        rh = Math.round(h * ratio);
                    }
                }
            }
            iuu.log('updateResizedSize', rw, rh);
            this.set({ resizedWidth: rw, resizedHeight: rh});
            this.set({ width: rw, height: rh}, { silent: false });
        },
        upload: function() {
            var self = this;
            var formData = new FormData();
            formData.append('__mode', 'iuu_upload');
            formData.append('blog_id', MultiUploadSettings.blog_id);
            formData.append('magic_token', MultiUploadSettings.magic_token);
            formData.append('edit_field', this.attributes.editField);
            formData.append('label', this.attributes.label);
            formData.append('thumb', this.attributes.thumb ? 1 : 0);
            formData.append('thumb_width', this.attributes.width);
            formData.append('thumb_height', this.attributes.height);
            formData.append('align', this.attributes.align);
            formData.append('popup', this.attributes.popup ? 1 : 0);
            formData.append('overwrite_yes', this.attributes.overwriteYes ? 1 : 0);
            formData.append('image_defaults', this.attributes.imageDefaults ? 1 : 0);
            formData.append('site_path', this.attributes.sitePath || '');
            formData.append('middle_path', this.attributes.middlePath || '');
            var destination = this.attributes.destination;
            if(this.attributes.customDestination) {
                destination = this.attributes.customDestination;
            }
            formData.append('destination', destination);
            formData.append('extra_path', this.attributes.extraPath || '');
            formData.append('image_upload_utility_watermark_insertion', this.attributes.watermarkInsertion ? 1 : 0);
            formData.append('file', this.attributes.file);

            // リサイズ機能を無効にする
            /*
            if(this.attributes.resize >= 0 && this.attributes.resize <= 2) {
                formData.append('enable_resize', this.attributes.resize);
                formData.append('max_size', this.attributes.resizeSize);
                formData.append('max_width', this.attributes.resizeWidth);
            }
            */

            this.lastError = null;
            this.lastResult = null;

            var promise = Backbone.$.ajaxq('UploadQueue', {
                url: MultiUploadSettings.mt_uri,
                type: 'post',
                data: formData,
                processData: false,
                contentType: false,
                success: function(data) {
                    iuu.log(self.cid + ' done: ', data);
                    if(data.error) {
                        self.lastError = data.error;
                        self.trigger('upload:file:failed', self);
                    }
                    else if(data.result) {
                        self.lastResult = data.result;
                        self.trigger('upload:file:success', self);
                    }
                    else {
                        self.lastError = '不明なエラーが発生しました。';
                        self.trigger('upload:file:failed', self);
                    }
                },
                error: function(jqXHR, textStatus/*, errorThrown */) {
                    iuu.log(self.cid + ' fail: ', textStatus);
                    self.lastError = '通信エラー (' + jqXHR.status + ' ' + jqXHR.statusText + ')';
                    self.trigger('upload:file:failed', self);
                }
            })
            .always(function() {
                iuu.log(self.cid + ' upload done');
                Backbone.trigger('upload:file:done', self);
            });
            return promise;
        }
    });
    return UploadFile;
});

