# ImageUploadUtility - (c) 2012-2017 Juxtaposition. All Rights Reserved.
# This code cannot be redistributed without permission from juxtaposition.jp
# For more information, consult your ImageUploadUtility license.
#

package ImageUploadUtility::CMS;

use strict;
use warnings;
use utf8;

use MT;
use MT::Asset;
use MT::CMS::Asset;
use Data::Dumper;
use Text::CSV_PP;

use constant DEBUG => 0;

sub _trace {
    my $msg = shift;
    my @info = caller(1);
    print STDERR "TRACE: $info[0]: $info[3]: $msg\n" if DEBUG;
}

sub watermark_insert {
    my $app = shift; 

    my $plugin = $app->component('ImageUploadUtility')
        or return $app->error("ImageUploadUtility plugin not found.");

    my $edit_field = $app->param('edit_field') || '';
    if ( !$edit_field 
        || $app->param('edit_field') !~ m/^watermark_asset$/ ) {
        return $app->permission_denied();
    }

    my $tmpl = $plugin->load_tmpl('watermark_insert.tmpl')
        or return $app->error($plugin->translate('Watermark insert template not found'));
    my $ctx = $tmpl->context;
    my $id = $app->param('id')
        or return $app->error($plugin->translate("Invalid request."));
    my $asset = MT::Asset->load($id);
    unless($asset) {
        return $app->error($plugin->translate('Asset not found, id:[_1]', $id));
    }
    unless($asset->file_ext =~ m!^(?:gif|jpg|png)$!) {
        return $app->error($plugin->translate('Invalid image type, type:[_1]', $asset->file_ext));
    }
    $ctx->stash('asset', $asset);
    $ctx->var('asset_id', $id);

    return $tmpl;
}

sub upload {
    my $app = shift;
    my $plugin = $app->component('ImageUploadUtility')
        or return $app->error("ImageUploadUtility plugin not found.");
    return $plugin->is_upload_compat ? _upload_compat($app, @_) : _upload($app, @_);
}

sub _upload_compat {
    my $app = shift;
    my $plugin = $app->component('ImageUploadUtility')
        or return $app->error("ImageUploadUtility plugin not found.");

    $app->param('dialog', 1);
    my $asset_select = 0;
    my $edit_field = $app->param('edit_field');
    # customfield_xxx
    if($edit_field =~ m!customfield_!) {
        $app->param('entry_insert', '');
        $app->param('asset_select', 1);
        $app->param('require_type', 'image');
        $asset_select = 1;
    }
    # editor-input-content|extended
    else { 
        $app->param('entry_insert', 1);
        $app->param('new_entry', 1);
        $app->param('include', 1);
    }

    my $overwrite_yes = $app->param('overwrite_yes') ? 1 : 0;
    my $overwrite_no = $overwrite_yes ? 0 : 1;
    $app->param->delete('overwrite_yes');
    $app->param->delete('overwrite_no');

    $app->param('wrap_text', 1);

    # my %param = $app->param_hash;
    # print STDERR Data::Dumper->Dump([ \%param ]);

    # upload file
    my $tmpl;
    UPLOAD_FILE: {
        $tmpl = MT::CMS::Asset::upload_file($app);
        if(!$tmpl && $app->errstr) {
            return $app->json_error($app->errstr);
        }
        unless(ref $tmpl && $tmpl->isa('MT::Template')) {
            return $app->json_error($plugin->translate('Unknown error occurred at upload_file: not template obj: ') . $tmpl);
        }
    }

    my $param = $tmpl->param();
    my $tmpl_name = $param->{template_filename} || '';
    if($tmpl_name eq 'dialog/asset_replace') {
        $app->param('temp', $param->{temp});
        $app->param('fname', $param->{fname});
        $app->param('overwrite_yes', $overwrite_yes);
        $app->param('overwrite_no', $overwrite_no);
        goto UPLOAD_FILE;
    }
    elsif($tmpl_name eq 'dialog/asset_upload') {
        my $errstr = $param->{error};
        if(!$errstr) {
            if($overwrite_no) {
                $errstr = $plugin->translate('Upload has canceled: file is duplicated.');
            }
            else {
                $errstr = $plugin->translate('Unknown error occurred at uploda_file: overwrite yes');
            }
        }
        return $app->json_error($errstr);
    }
    
    $app->param('id', $param->{asset_id});

    if(!$asset_select) {
        # insert asset
        $tmpl = MT::CMS::Asset::insert($app);
        if(!$tmpl && $app->errstr) {
            return $app->json_error($app->errstr);
        }
        unless(ref $tmpl && $tmpl->isa('MT::Template')) {
            my $errstr = $plugin->translate('Unknown error occurred at insert_asset: not template obj: ') . $tmpl;
            return $app->json_error($errstr);
        }
    }

    $param = $tmpl->param();
    my $ctx = $tmpl->context;
    my $asset = $ctx->stash('asset');
    unless($asset) {
        my $assets = $ctx->stash('assets');
        unless($assets || ref $asset eq 'ARRAY') {
            return $app->json_error($plugin->translate('Asset object not found.'));
        }
        $asset = $assets->[0];
        unless($asset->isa('MT::Asset')) {
            return $app->json_error($plugin->translate('Invalid asset object.'));
        }
    }
    my $preview = '';
    # customfield
    if($asset_select) {
        require MT::Util;
        my $title = MT::Util::encode_js($app->translate("View image"));
        my $asset_url = $asset->url;
        my ($thumbnail_url) = $asset->thumbnail_url(Width => 240, Height => 240);
        $preview = <<HTML;
<a href="$asset_url" target="_blank" title="$title"><img src="$thumbnail_url" alt="" /></a>
HTML
    }

    # Asset List Item 
    my $asset_id = $asset->id;
    my $asset_class = $asset->class;
    my $asset_file_name = $asset->file_name;
    my ($mouseevent, $thumbnail) = ('', '');
    if($asset_class eq 'image') {
        $mouseevent = qq{onmouseover="show('list-image-$asset_id')" onmouseout="hide('list-image-$asset_id')"};
        my ($list_item_thumbnail_url) = $asset->thumbnail_url(Width => 100);
        $thumbnail = qq{<img id="list-image-$asset_id" src="$list_item_thumbnail_url" class="list-image hidden" />};
    }
    my $script_url = $app->uri;
    my $blog_id = $app->param('blog_id');
    my $asset_list_item = <<HTML;
<li id="list-asset-$asset_id" class="asset-type-$asset_class" $mouseevent>
<a href="$script_url?__mode=view&_type=asset&blog_id=$blog_id&id=$asset_id" class="asset-title">$asset_file_name</a>
<a href="javascript:removeAssetFromList($asset_id)" class="remove-asset"></a>
$thumbnail
</li>
HTML

    my $res = {
        content => $param->{upload_html} || '',
        edit_field => scalar $app->param('edit_field') || '',
        asset_type => $asset->class,
        asset_id => $asset->id,
        preview => $preview,
        asset_list_item => $asset_list_item,
    };
    return $app->json_result($res);
}

sub _upload {
    my $app = shift;
    my $plugin = $app->component('ImageUploadUtility')
        or return $app->error("ImageUploadUtility plugin not found.");

    $app->param('dialog', 1);
    my $asset_select = 0;
    my $edit_field = $app->param('edit_field');
    my $content_field_id;
    # customfield_xxx
    if($edit_field =~ m!customfield_!) {
        $app->param('entry_insert', '');
        $app->param('asset_select', 1);
        $app->param('require_type', 'image');
        $asset_select = 1;
    }
    # asset-field-xxx
    elsif($edit_field =~ m!asset-field-(\d+)!) {
        $app->param('entry_insert', '');
        $app->param('asset_select', 2);
        $app->param('require_type', 'image');
        $asset_select = 2;
        $content_field_id = $1;
    }
    # editor-input-content|extended
    else { 
        $app->param('entry_insert', 1);
        $app->param('new_entry', 1);
        $app->param('include', 1);
    }

    my $overwrite_yes = $app->param('overwrite_yes') ? 1 : 0;
    $app->param('operation_if_exists', $overwrite_yes ? 2 : 3);

    $app->param('wrap_text', 1);

     my %param = $app->param_hash;
    my $asset;
    {
        no strict 'refs';
        no warnings 'redefine';
        my ($result, $error);
        my $saved_json_result = \&MT::App::json_result;
        local *MT::App::json_result = sub {
            ($app, $result) = @_;
            return;
        };
        my $saved_json_error = \&MT::App::json_error;
        local *MT::App::json_error = sub {
            ($app, $error) = @_;
            return &$saved_json_error(@_);
        };

        # upload file
        MT::CMS::Asset::js_upload_file($app);
        return if $error;
        return if !$error && !$result;  # no asset;
        return unless $result;
        if($result && exists $result->{cancel}) {
            return &$saved_json_error($app, $result->{cancel});
        }
        my $asset_id = $result->{asset}->{id}
            or return &$saved_json_error($app, $plugin->translate('Asset ID not found.'));
        $asset = MT::Asset->load($asset_id)
            or return $app->json_error($plugin->translate('Asset not found.'));
        $app->param('id', $asset_id);
    }

    my $param;
    my $upload_html = '';
    my $preview = '';
    if($asset_select == 0) {
        # insert asset
        my $tmpl = MT::CMS::Asset::insert($app);
        if(!$tmpl && $app->errstr) {
            return $app->json_error($app->errstr);
        }
        unless(ref $tmpl && $tmpl->isa('MT::Template')) {
            my $errstr = $plugin->translate('Unknown error occurred at insert_asset: not template obj: ') . $tmpl;
            return $app->json_error($errstr);
        }

        $param = $tmpl->param();
        $upload_html = $param->{upload_html} || '';
    }
    elsif($asset_select == 1) {
        # customfield
        $upload_html = $asset->as_html({ 
            wrap_text => 1,
            enclose => 1,
        });
        require MT::Util;
        my $title = MT::Util::encode_js($app->translate("View image"));
        my $asset_url = $asset->url;
        my ($thumbnail_url) = $asset->thumbnail_url(Width => 240, Height => 240);
        $preview = <<HTML;
<a href="$asset_url" target="_blank" title="$title"><img src="$thumbnail_url" alt="" /></a>
HTML
    }
    elsif($asset_select == 2) {
        # asset field
        $app->log('asset_select: ' . $asset_select);
        my $options;
        my $can_multi;
        require MT::ContentField;
        my $cf = MT::ContentField->load($content_field_id)
            or return $app->json_error("Unknown upload request, edit_field=$edit_field");
        $options = $cf->options;
        $can_multi = $options->{multiple} ? 1 : 0;

        require MT::CMS::Asset;
        my $hasher = MT::CMS::Asset::build_asset_hasher(
            $app,
            PreviewWidth => $options->{preview_width} || 80,
            PreviewHeight => $options->{preview_height} || 80,
        );
        my $row = $asset->get_values;
        $hasher->($asset, $row);
        my $asset_data = {
            asset_dimensions     => $row->{'Actual Dimensions'},
            asset_file_name      => $row->{file_name},
            asset_id             => $row->{id},
            asset_label          => $row->{label},
            asset_preview_url    => $row->{preview_url},
            asset_preview_height => $row->{preview_height},
            asset_preview_width  => $row->{preview_width},
            asset_type           => $row->{class},
        };
        $app->log('asset_data: ' . Data::Dumper->Dump([$asset_data]));

        my $callback = sub {
            my ($cb, $app, $tmpl_ref, $file) = @_;
            return 1 unless $app->param('asset_select') == 2;
            $app->log('_upload: template_souece.field_html_asset: asset_select=2');
            $$tmpl_ref =~ s!^.+?(<mt:loop\sname="asset_loop">.+?</mt:loop>).+$!$1!gxms;
            return 1;
        };
        my $cb = $app->add_callback('template_source.field_html_asset', 5, $plugin, $callback);
        $upload_html = $app->build_page('field_html/field_html_asset.tmpl', {
            asset_loop => [ $asset_data ],
            multiple => $can_multi,
            content_field_id => $content_field_id,
        });
        $app->remove_callback($cb);
        $app->log('upload_html: ' . $upload_html);
    }
    else {
        return $app->json_error("Unknown upload request, asset_select=$asset_select");
    }

    # Asset List Item 
    my $asset_id = $asset->id;
    my $asset_class = $asset->class;
    my $asset_file_name = $asset->file_name;
    my ($mouseevent, $thumbnail) = ('', '');
    if($asset_class eq 'image') {
        $mouseevent = qq{onmouseover="show('list-image-$asset_id')" onmouseout="hide('list-image-$asset_id')"};
        my ($list_item_thumbnail_url) = $asset->thumbnail_url(Width => 100);
        $thumbnail = qq{<img id="list-image-$asset_id" src="$list_item_thumbnail_url" class="list-image hidden" />};
    }
    my $script_url = $app->uri;
    my $blog_id = $app->param('blog_id');
    my $asset_list_item = <<HTML;
<li id="list-asset-$asset_id" class="asset-type-$asset_class" $mouseevent>
<a href="$script_url?__mode=view&_type=asset&blog_id=$blog_id&id=$asset_id" class="asset-title">$asset_file_name</a>
<a href="javascript:removeAssetFromList($asset_id)" class="remove-asset"></a>
$thumbnail
</li>
HTML

    my $res = {
        content => $upload_html,
        edit_field => scalar $app->param('edit_field') || '',
        asset_type => $asset->class,
        asset_id => $asset->id,
        preview => $preview,
        asset_list_item => $asset_list_item,
    };
    return $app->json_result($res);
}

sub image_defaults {
    my $app = shift;
    my $plugin = $app->component('ImageUploadUtility')
        or return $app->json_error('ImageUploadUtility plugin not found.');
    my $blog = $app->blog
        or return $app->json_error('Blog not found');
    my $params = {};
    my $perms = $app->permissions;
    $params->{can_save_image_defaults}
        = $perms->can_do('save_image_defaults') ? 1 : 0;
    $params->{popup} = $blog->image_default_popup ? 1 : 0;
    $params->{make_thumb} = $blog->image_default_thumb ? 1 : 0;
    $params->{ 'align_' . $_ } 
        = ( $blog->image_default_align || 'none' ) eq $_ ? 1 : 0
        for qw(none left center right);
    $params->{thumb_width} = $blog->image_default_width || 0;

    return $app->json_result($params);
}

sub get_plugin_param_names {
    my ($settings) = @_;
    my @names;
    return @names unless $settings;
    foreach my $s (@$settings) {
        my ($name, $values) = @$s;
        $name =~ s!^image_upload_utility_!!;
        next if defined $values->{scope} && $values->{scope} eq 'system';
        push @names, $name;
    }
    return \@names;
}

sub download_setting {
    my $app = shift;
    my $plugin = $app->component('ImageUploadUtility');

    $app->validate_magic or return;

    my $enc = $app->config('ExportEncoding');
    $enc = $app->config('LogExportEncoding') if ( !$enc );
    $enc = $app->charset || $app->config->PublishCharset unless $enc;

    my @ts = localtime(time);
    my $ts = sprintf("%04d%02d%02d%02d%02d%02d", $ts[5]+1900, $ts[4]+1, @ts[3,2,1,0]);
    my $filename = "ImageUploadUtility-Settings-$ts.csv";

    my $csv = Text::CSV_PP->new({ binary => 1 }) 
        or return $app->error($plugin->translate('Failed to create Text Parser.'));

    local $| = 1;
    $app->{no_print_body} = 1;
    $app->set_header('Content-Disposition' => "attachment; filename=$filename");
    $app->send_http_header($enc ? "text/csv; charset=$enc" : 'text/csv');

    my $s = $plugin->settings;
    my $param_names = get_plugin_param_names($s);
    my $param_names_str = join(",", @$param_names);

    my @lines = ();
    my $line = "id,name";
    if($param_names_str) {
        $line .= "," . $param_names_str;
    }
    $line .= "\n";

    require MT::Blog; 
    my $iter = MT::Blog->load_iter({ class => '*' });
    while(my $blog = $iter->()) {
        my $config = {};
        $plugin->load_config($config, 'blog:'.$blog->id);
        my @cols = ();
        push @cols, $blog->id;
        push @cols, $blog->name;
        foreach my $name (@$param_names) {
            push @cols, $config->{'image_upload_utility_' . $name};
        }
        unless(my $status = $csv->combine(@cols)) {
            $app->log($plugin->translate('Failed to combine columns.'));
            next;
        }
        $line .= $csv->string() . "\n";
        $app->print( Encode::encode( $enc, $line ) );
        $line = '';
    }
}

sub upload_setting {
    my $app = shift;
    my $plugin = $app->component('ImageUploadUtility');

    $app->validate_magic or return;

    my $enc = $app->config('ExportEncoding');
    $enc = $app->config('LogExportEncoding') if ( !$enc );
    $enc = $app->charset || $app->config->PublishCharset unless $enc;

    my ($fh, $info) = $app->upload_info('file');
    unless($fh) {
        return $app->error($plugin->translate('File is required.'));
    }
    # skip header
    my $header = <$fh>;
    
    my $s = $plugin->settings;
    my $param_names = get_plugin_param_names($s);
    my $num_names = scalar @$param_names;

    my $csv = Text::CSV_PP->new({ binary => 1, empty_is_undef => 1 })
        or return $app->error($plugin->translate('Failed to create Text Parser.'));

    my $error;
    my $line_no = 2;
    while(my $line = <$fh>) {
        next unless $line;
        $line = Encode::decode($enc, $line);
        unless(my $status = $csv->parse($line)) {
            return $app->error($plugin->translate('Failed to parse the text.'));
        }
        my @cols = $csv->fields();
        my $id = int($cols[0]);
        unless($id) {
            $app->log($plugin->translate('Invalid blog id, line=[_1]', $line_no));
            $error = 1;
            next;
        }
        require MT::Blog;
        unless(my $blog = MT::Blog->load($id)) {
            $app->log($plugin->translate('Blog does not exists. ID=[_1], line=[_2]', $id, $line_no));
            $error = 1;
            next;
        }

        my $name = $cols[1];

        my $config = {};
        my $offset = 2; # skip id, name
        for(my $i=0; $i<$num_names; $i++) {
            my $name = $param_names->[$i];
            my $value = $cols[$i+$offset];
            if(defined $value) {
                $config->{'image_upload_utility_' . $name} = $value;
            }
        }
        eval {
            $plugin->apply_default_settings($config, 'blog:'.$id);
            $plugin->save_config($config, 'blog:'.$id);
        };
        if($@) {
            $app->log($plugin->translate('Failed to save plugin config, ID=[_1], line=[_2]', $id, $line_no));
        }
    }
    continue {
        $line_no++;
    }

    if($error) {
        $app->add_return_arg('bulk_setting_error', 1);
    }
    else {
        $app->add_return_arg('saved', 1);
    }
    $app->call_return;
}



1;
__END__


