# ImageUploadUtility - (c) 2012-2017 Juxtaposition. All Rights Reserved.
# This code cannot be redistributed without permission from juxtaposition.jp
# For more information, consult your ImageUploadUtility license.
#

package ImageUploadUtility::Plugin::Extra;

use strict;
use warnings;
use utf8;
use Carp;
use Data::Dumper;
use Capture::Tiny qw( capture );
use MT::Asset::PDF;

use constant DEBUG => 0;

sub _merge_hash {
    my ($h1, $h2) = @_;
    $h1 = {} unless defined $h1;
    $h2 = {} unless defined $h2;
    my %h = ( %$h1, %$h2 );
    return \%h;
}

sub merge_registry {
    my ($plugin, $registry) = @_;

    $registry->{object_types} = _merge_hash(
        $registry->{object_types}, 
        { 
            'asset.pdf' => 'MT::Asset::PDF',
            'pdf' => 'MT::Asset::PDF',
        }         
    );

    $registry->{applications}->{cms}->{methods} = _merge_hash( 
        $registry->{applications}->{cms}->{methods}, 
        { 
            watermark_insert => '$ImageUploadUtility::ImageUploadUtility::CMS::watermark_insert',
            iuu_upload => '$ImageUploadUtility::ImageUploadUtility::CMS::upload',
            iuu_image_defaults => '$ImageUploadUtility::ImageUploadUtility::CMS::image_defaults',
            iuu_download_setting => '$ImageUploadUtility::ImageUploadUtility::CMS::download_setting',
            iuu_upload_setting => '$ImageUploadUtility::ImageUploadUtility::CMS::upload_setting',
        }         
    );

    $registry->{applications}->{cms}->{callbacks} = _merge_hash(
        $registry->{applications}->{cms}->{callbacks},
        {
            'template_source.asset_list' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::source_asset_list',
            'template_param.asset_list' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_asset_list',
            'template_source.asset_upload' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::source_asset_upload',
            'template_source.asset_replace' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::source_asset_replace',
            'template_param.asset_replace' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_asset_replace',
            'template_source.asset_options' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::source_asset_options',
            'template_source.multi_asset_options' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::source_asset_options',
            'template_param.asset_options' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_asset_options',
            'template_param.asset_options_image' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_asset_options_image',
            'template_param.insert_options_image' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_insert_options_image',
            'template_param.edit_field' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_edit_field',
            'cms_pre_save.asset' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::cms_pre_save_asset',
            'app_pre_listing_dialog_list_asset' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::pre_listing_dialog_list_asset',
            'cms_filtered_list_param.asset' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::filtered_list_param_asset',
            'template_param.asset_insert' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::param_asset_insert',
            'template_source.edit_asset' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::source_edit_asset',
            'cms_upload_file' => '$ImageUploadUtility::ImageUploadUtility::Plugin::Extra::cms_upload_file',
        }
    );

    $registry->{editors} = _merge_hash(
        $registry->{editors},
        {
            tinymce => {
                extension => 'extension.tmpl',
            },
        }
    );

    return $registry;
}

sub add_watermark {
    my ($app, $magick, $watermark_asset_id) = @_;

    croak 'Missing args, magick' unless $magick;
    croak 'Missing args, watermark_asset_id' unless $watermark_asset_id;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";

    # control watermark on/off at file upload dialog
    my $blog_id = $app->param('blog_id') || 0;
    my $watermark_insertion = $app->param('image_upload_utility_watermark_insertion') || 0;
    my $always_use = $plugin->watermark_always_use($blog_id);
    if(!$always_use && !$watermark_insertion) {
        warn "add_watermark: always_use=$always_use, watermark_insertion=$watermark_insertion" if DEBUG;
        return 1;
    }

    require MT::Asset;
    my $watermark_asset = MT::Asset->load($watermark_asset_id);
    if ($watermark_asset) {
        require MT::Image;
        my $watermark_img = MT::Image->new( Filename => $watermark_asset->file_path );
        if ($watermark_img) {
            my $watermark_magick = $watermark_img->can('_magick')
                ? $watermark_img->_magick
                : (exists $watermark_img->{magick} ? $watermark_img->{magick} : undef);
            my $gravity = $app->config->IUUWatermarkPosition
                || $plugin->watermark_position($blog_id) || 'SouthEast';
            my $opacity = $app->config->IUUWatermarkOpacity
                || $plugin->watermark_opacity($blog_id) || 100;
            my $rc = $magick->Composite(
                image => $watermark_magick,
                gravity => $gravity,
                compose => 'Dissolve',
                opacity => "$opacity%",
            );
            return 1 unless "$rc";
        }
    }

    return;
}

sub init_app {
    my ($cb, $app) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    my $blog_id = $app->can('param') ? $app->param('blog_id') || 0 : 0;

    # Replace complete_insert
    no strict 'refs';
    no warnings 'redefine';
    require MT::CMS::Asset;
    my $saved_complete_insert = \&MT::CMS::Asset::complete_insert;
    my $_complete_insert = sub {
        my $app  = shift;
        my (%args) = @_;
        my $tmpl = $saved_complete_insert->($app, %args)
            or return;
        my $param = $tmpl->param;
        my $asset = $args{asset}
            or return $tmpl;
        unless ( $asset->file_ext eq 'pdf' ) {
            return $tmpl;
        }
        my $enable_pdf_thumbnail = $plugin->enable_pdf_thumbnail
            or return $tmpl;

        my $perms = $app->{perms};
        my $blog = $asset->blog
            or return $tmpl;
        $param->{do_thumb} = 1;
        $param->{can_save_image_defaults}
            = $perms->can_do('save_image_defaults') ? 1: 0;
        $param->{popup}      = $blog->image_default_popup     ? 1 : 0;
        $param->{wrap_text}  = $blog->image_default_wrap_text ? 1 : 0;
        $param->{make_thumb} = $blog->image_default_thumb     ? 1 : 0;
        $param->{ 'align_' . $_ }
            = ( $blog->image_default_align || 'none' ) eq $_ ? 1 : 0
            for qw(none left center right);
        $param->{ 'unit_w' . $_ }
            = ( $blog->image_default_wunits || 'pixels' ) eq $_ ? 1 : 0
            for qw(percent pixels);
        my ($w, $h) = ($asset->image_width, $asset->image_height);
        unless ( defined $w && defined $h ) {
            return $tmpl;
        }
        $param->{width} = $w || 0;
        $param->{height} = $h || 0;
        $param->{thumb_width} = $blog->image_default_width || $w || 0;
        # $param->{thumb_height} = $blog->image_default_height || $h || 0;
    
        my $html = $app->build_page( 'dialog/asset_options_image.tmpl', $param );

        $tmpl->param(options_snippet => $html);

        return $tmpl;
    };
    *MT::CMS::Asset::complete_insert = $_complete_insert;

    # Replace insert
    my $saved_insert = \&MT::CMS::Asset::insert;
    my $_insert = sub {
        my $app = shift;
        no strict 'refs';
        no warnings 'redefine';
        my $saved_process_post_upload = \&MT::CMS::Asset::_process_post_upload;
        local *MT::CMS::Asset::_process_post_upload = sub {
            my $app = shift;

            my $output = $saved_process_post_upload->($app);

            my %param = $app->param_hash;
            $app->log("param: " . Data::Dumper->Dump([ \%param ])) if DEBUG;

            require MT::Asset;
            require MT::Asset::Image;
            my $id = $param{id}
                or return $output;
            my $asset = MT::Asset->load( $id )
                or return $output;
            my $enable_pdf_thumbnail = $plugin->enable_pdf_thumbnail;
            if ( $enable_pdf_thumbnail && $asset->class eq 'pdf' && $asset->file_ext eq 'pdf' ) {
                $output = create_thumbnail_link($asset, \%param);
            }
            return $output;
        };
        if($app->param('edit_field')
            && $app->param('edit_field') =~ m!^watermark_asset$!) {
            require ImageUploadUtility::CMS;
            return ImageUploadUtility::CMS::watermark_insert($app);
        }
        return $saved_insert->($app);
    };
    *MT::CMS::Asset::insert = $_insert;

    # Replace multiupload insert
    unless ($plugin->is_upload_compat) {
        my $saved_insert_asset = \&MT::CMS::Asset::insert_asset;
        my $_insert_asset = sub {
            my $app = shift;
            my $asset_map = {};
            $app->request('iuu_asset_map', $asset_map);
            my $tmpl = $saved_insert_asset->($app, @_);
            $app->request('iuu_asset_map', undef);

            if (!$app->param('no_insert') && !$app->param('direct_asset_insert')) {
                my $prefs = $app->param('prefs_json');
                $prefs =~ s/^"|"$//g;
                $prefs =~ s/\\//g;
                $prefs = eval { MT::Util::from_json($prefs) };
                if ( !$prefs ) {
                    return $app->errtrans('Invalid request.');
                }

                my $blog_id = $app->param('blog_id') || 0;
                my $text = '';
                foreach my $item (@$prefs) {
                    my $id = $item->{id}
                        or return $tmpl;
                    my $asset = MT::Asset->load($id)
                        or return $tmpl;
                    my %param;
                    foreach my $k ( keys %$item ) {
                        my $name = $k;
                        if ( $k =~ m/(.*)[-|_]$id/ig ) {
                            $name = $1;
                        }
                        $param{$name} = $item->{$k};
                    }
                    $param{wrap_text} = 1;
                    $param{new_entry} = $app->param('new_entry') ? 1 : 0;

                    my $out;
                    my $enable_pdf_thumbnail = $plugin->enable_pdf_thumbnail;
                    if ( $enable_pdf_thumbnail && $asset->class eq 'pdf' && $asset->file_ext eq 'pdf' ) {
                        $out = create_thumbnail_link($asset, \%param)
                            or return $tmpl;
                    }
                    else {
                        my $pkg = MT::Asset->handler_for_file($asset->file_name);
                        my $a = $pkg eq 'MT::Asset::PDF' ? $asset : bless $asset, $pkg;
                        if($asset_map->{$a->id}->{thumbnail}) {
                            $param{thumb_asset_id} = $asset_map->{$a->id}->{thumbnail};
                        }
                        if($asset_map->{$a->id}->{popup}) {
                            $param{popup_asset_id} = $asset_map->{$a->id}->{popup};
                        }
                        $out = $a->as_html( \%param )
                            or return $tmpl;
                    }
                    $text .= $out;
                }
                $tmpl->param({ 
                    upload_html => $text || '',
                });
            }
            return $tmpl;
        };
        *MT::CMS::Asset::insert_asset = $_insert_asset;
    }

    # Replace protected pdf thumbnail
    my $saved_make_thumbnail_url = \&MT::CMS::Asset::_make_thumbnail_url;
    my $_make_thumbnail_url = sub {
        my $asset = shift;
        my $thumb_url = &$saved_make_thumbnail_url($asset, @_);
        if($thumb_url =~ m!pdf-45.png$!) {
            $thumb_url =~ s!pdf-45!file-45!;
        }
        return $thumb_url;
    };
    *MT::CMS::Asset::_make_thumbnail_url = $_make_thumbnail_url;

    return 1;
}

sub rebless_asset_if_pdf {
    my ($asset) = shift;
    my $plugin = MT->instance()->component('ImageUploadUtility');
    if($asset
        && $asset->isa('MT::Asset')
        && !$asset->isa('MT::Asset::PDF')
        && $asset->class eq 'pdf'
        && $asset->file_ext eq 'pdf'
        && $plugin->enable_pdf_thumbnail) {
        require MT::Asset::PDF;
        # rebless to MT::Asset::PDF
        bless $asset, 'MT::Asset::PDF';
    }
    return $asset;
}

sub create_thumbnail_link {
    my ($asset, $param) = @_;
    
    my $orig_asset = $asset->clone;
    my $app = MT->instance;

    # bless to pdf
    require MT::Asset::PDF;
    bless $asset, 'MT::Asset::PDF';

    # copy params
    my %_param = %$param;
    $_param{thumb} = 1;

    unless($asset->on_upload(\%_param)) {
        $app->log("error occurred in on_upload") if DEBUG;
        return;
    }

    if(defined $_param{edit_field}) {
        $_param{enclose} = $_param{edit_field} =~ /^customfield/ ? 1 : 0;
    }
    $app->log('create_pdf_thumbnail: ' . Data::Dumper->Dump([ \%_param ])) if DEBUG;
    if($asset->is_protected) {
        $_param{include} = 0;
    }
    my $out = $asset->as_html(\%_param);
    $app->log('create_pdf_thumbnail: out: ' . $out) if DEBUG;

    # rebless to original class
    bless $asset, ref $orig_asset;

    return $out;
}

sub get_size {
    my (%params) = @_;

    my $app = MT->instance
        or die 'Failed to get MT instance.';
    my $plugin = $app->component('ImageUploadUtility')
        or die 'Failed to get plugin instance.';

    my $graphic_drv = $plugin->get_image_driver;
    my $img = $graphic_drv->new;
    unless($img) {
        $app->log("failed to create $graphic_drv instance");
        die "failed to create $graphic_drv instance";
    }

    # Redirect STDOUT & STDERR because PDF read error is written to there.
    my $err;
    my ( $stdout, $stderr );
    if($params{Filename}) {
        ( $stdout, $stderr ) = capture {
            $err = $img->Read($params{Filename} . '[0]');
        };
    }
    elsif($params{Fh}) {
        my $fh = $params{Fh};
        seek $fh, 0, 0;
        #( $stdout, $stderr, $err ) = capture {
            eval {
                $err = $img->Read(file => $fh);
            };
            die $@ if $@;
        #    return $@ if $@;
        #    return $err;
        #};
        seek $fh, 0, 0;
    }
    else {
        $app->log("Failed to get input source.")
            if $app->config('DebugMode'); 
        return;
    }

    if ( "$err" || $stderr ) {
        $app->log(
            $plugin->translate("Failed to get size: [_1]: [_2]", $params{Filename} || $params{Fh}, $err . ', ' . $stderr)
        );
        return;
    }

    my ( $w, $h ) = $img->Get( 'width', 'height' );
    return ( $w, $h );
}

sub _asset_dimension {
    my ($asset) = @_;
    my $asset_pkg = ref $asset;
    require MT::Asset::PDF;
    bless $asset, 'MT::Asset::PDF';
    # my $meta = $asset->metadata;
    my ($w, $h) = ($asset->image_width, $asset->image_height);
    bless $asset, $asset_pkg;
    # return ($meta->{image_width}, $meta->{image_height});
    return ($w, $h);
}

#
# Transformer
#
sub source_asset_list {
    my ($cb, $app, $tmpl_ref, $file) = @_;

    my $next_mode = $app->param('next_mode');
    unless(defined $next_mode && $next_mode eq 'watermark_insert') {
        return 1;
    }
    
    return 1;

    my $old = quotemeta(<<'HTML');
      <mt:var name="upload_new_file_link">
HTML
    $$tmpl_ref =~ s!$old!!;

    return 1;
}

sub param_asset_list {
    my ($cb, $app, $param, $tmpl) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak 'Plugin not found.'; 
    my $blog_id = $app->param('blog_id') || 0;
    $param->{image_upload_utility_watermark_always_use} = $plugin->watermark_always_use($blog_id);

    return 1;
}

sub source_asset_upload {
    my ($cb, $app, $tmpl_ref, $file) = @_;

    my $pattern = quotemeta(<<'HTML');
  <mt:if name="enable_destination">
  <mt:var name="container_type" value="folder">
  <mt:include name="include/category_selector.tmpl">
  </mt:if>
HTML
    my $replacement = <<HTML;
<mt:unless name="edit_field" eq="watermark_asset">
<mt:unless name="image_upload_utility_watermark_always_use">
<__trans_section component="ImageUploadUtility">
<mtapp:setting
    id="image_upload_utility_watermark_insertion"
    label_class="top-label"
    label="<__trans phrase="Watermark" />">
    <input type="checkbox" id="image_upload_utility_watermark_insertion" name="image_upload_utility_watermark_insertion" value="1" />
    <label for="image_upload_utility_watermark_insertion"><__trans phrase="Insert wartermark" /></label>
</mtapp:setting>
</__trans_section>
<script type="text/javascript">
jQuery('#file').change(function(){
    if(jQuery(this).val().match(/.+\.(?:jpg|gif|png)/i)) {
        jQuery('#image_upload_utility_watermark_insertion-field').show();
    }
    else {
        jQuery('#image_upload_utility_watermark_insertion-field').hide();
    }
});
</script>
</mt:unless>
<mt:else>
<__trans_section component="ImageUploadUtility">
<script type="text/javascript">
jQuery('#file').change(function(){
    if(!jQuery(this).val().match(/.+\.(?:jpg|gif|png)/i)) {
        alert('<__trans phrase="Select a JPEG/GIF/PNG image." />');
        jQuery(this).val('');
    }
});
</script>
</__trans_section>
</mt:unless>
HTML
    $$tmpl_ref =~ s!($pattern)!$1$replacement!;

    return 1;
}

sub source_asset_replace {
    my ($cb, $app, $tmpl_ref, $file) = @_;

    my $pattern = quotemeta(<<'HTML');
  <input type="hidden" name="overwrite_no" value="0" />
HTML
    my $replacement = <<HTML;
  <input type="hidden" name="image_upload_utility_watermark_insertion" value="<mt:var name="image_upload_utility_watermark_insertion" />" />
HTML
    $$tmpl_ref =~ s!($pattern)!$1$replacement!;

    return 1;
}

sub param_asset_replace {
    my ($cb, $app, $param, $tmpl) = @_;

    $param->{image_upload_utility_watermark_insertion} = $app->param('image_upload_utility_watermark_insertion') || '0';

    return 1;
}

sub source_asset_options {
    my ($cb, $app, $tmpl_ref, $file) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    return 1 unless $plugin->enable_pdf_thumbnail;

    my $pattern = quotemeta(<<'HTML');
<mt:include name="dialog/footer.tmpl">
HTML
    my $replacement = <<'HTML';
<mt:if name="image_upload_utility_enable_pdf_thumbnail">
<mt:setvarblock name="jq_js_include" append="1">
    var $thumbnail = jQuery('#create_thumbnail');
    $thumbnail.click(function() { return false; });
    if(!$thumbnail.attr('checked')) {
        $thumbnail.attr('checked', 'checked');
    }
</mt:setvarblock>
</mt:if>
HTML
    $$tmpl_ref =~ s!($pattern)!$replacement$1!;

    return 1;
}

sub param_asset_options {
    my ($cb, $app, $param, $tmpl) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    return 1 unless $plugin->enable_pdf_thumbnail;

    require File::Basename;
    my ( $base, $path, $ext )
        = File::Basename::fileparse($param->{fname}, qr/[A-Za-z0-9]+$/);
    if(defined $ext && $ext eq 'pdf') { 
        $param->{image_upload_utility_enable_pdf_thumbnail} = 1;
    }
    return 1;
}

sub param_asset_options_image {
    my ($cb, $app, $param, $tmpl) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    return 1 unless $plugin->enable_pdf_thumbnail;

    require File::Basename;
    my ( $base, $path, $ext )
        = File::Basename::fileparse($param->{fname}, qr/[A-Za-z0-9]+$/);
    if(defined $ext && $ext eq 'pdf') { 
        my $plugin = $app->component('ImageUploadUtility')
            or croak "Plugin not found.";
        # label
        my $node = $tmpl->getElementById('link_to_popup');
        $node->setAttribute('label', '<__trans_section component="ImageUploadUtility"><__trans phrase="Link to PDF file in a new window(tab)."></__trans_section>');
        # checkbox label
        my $innerHTML = $node->innerHTML();
        $innerHTML =~ s{<__trans phrase="Link image to full-size version in a popup window.">}
                       {<__trans_section component="ImageUploadUtility"><__trans phrase="Link to PDF file in a new window(tab)."></__trans_section>};
        $node->innerHTML($innerHTML);
    }
    
    return 1;
}

sub param_insert_options_image {
    my ($cb, $app, $param, $tmpl) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    return 1 unless $plugin->enable_pdf_thumbnail;

    require File::Basename;
    my ( $base, $path, $ext )
        = File::Basename::fileparse($param->{filename}, qr/[A-Za-z0-9]+$/);
    if(defined $ext && $ext eq 'pdf') { 
        my $plugin = $app->component('ImageUploadUtility')
            or croak "Plugin not found.";
        # thumb
        $param->{do_thumb} = 1;
        $param->{make_thumb} = 1;
        # label
        my $node = $tmpl->getElementById('link_to_popup-<mt:var name="id">');
        $node->setAttribute('label', '<__trans_section component="ImageUploadUtility"><__trans phrase="Link to PDF file in a new window(tab)."></__trans_section>');
        # checkbox label
        my $innerHTML = $node->innerHTML();
        $innerHTML =~ s{<__trans phrase="Link image to full-size version in a popup window.">}
                       {<__trans_section component="ImageUploadUtility"><__trans phrase="Link to PDF file in a new window(tab)."></__trans_section>};
        $node->innerHTML($innerHTML);
        my $id = $param->{id};
        my $js = <<"HTML";
<script type="text/javascript">
jQuery(function(){
    var thumbnail = jQuery('#create_thumbnail-$id');
    thumbnail.click(function() { return false; });
    if(!thumbnail.attr('checked')) {
        thumbnail.attr('checked', 'checked');
    }
});
</script>
HTML
        my $js_node = $tmpl->createTextNode($js);
        $js_node->innerHTML($js);
        $tmpl->insertAfter($js_node, $node);
    }
    
    return 1;
}


sub cms_pre_save_asset {
    my ($cb, $app, $asset, $original) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    return 1 unless $plugin->enable_pdf_thumbnail;

    my $blog_id = $app->can('param') ? $app->param('blog_id') : 0;
    return 1 unless $asset->file_ext;

    my $ext = $asset->file_ext || '';
    $ext = lc($ext);
    if($ext eq 'pdf' && !$asset->isa('MT::Asset::PDF')) {
        $asset->class('file');
        # require MT::Asset::PDF;
        # bless $asset, 'MT::Asset::PDF';
        # $asset->{__meta}->{pkg} = 'MT::Asset::PDF';
        # $asset->class('pdf');
    }
    elsif($asset->class eq 'pdf' && $ext ne 'pdf') {
        my $pkg = MT::Asset->handler_for_file($asset->file_name); 
        $asset->class($pkg->class_type);
    }

    if($asset->isa('MT::Asset::PDF')) {
        my ($w, $h) = ($asset->image_width, $asset->image_height); 
    }

    return 1;
}

sub pre_listing_dialog_list_asset {
    my ($cb, $app, $terms, $args, $param, $hasher_ref) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak "Plugin not found.";
    return 1 unless $plugin->enable_pdf_thumbnail;

    my $blog_id = $app->can('param') ? $app->param('blog_id') : 0;

    if(exists $terms->{class} && ref $terms->{class} eq 'ARRAY') {
        push @{$terms->{class}}, 'pdf';
    }

    my $hasher = $$hasher_ref;
    my $wrapper = sub {
        my ( $obj, $row, %param ) = @_;
        # require MT::Asset::PDF;
        if($obj->class eq 'pdf' && !$obj->isa('MT::Asset::PDF')) {
            bless $obj, 'MT::Asset::PDF';
        }
        return $hasher->(@_);
    };
    $$hasher_ref = $wrapper;

    return 1;
}

sub toggle_asset_type {
    my ($plugin, $value) = @_;
    my $driver = MT::Asset->driver
        or return;
    my $type = $value ? 'pdf' : 'file';
    $driver->sql(<<SQL);
UPDATE mt_asset
SET asset_class = '$type'
WHERE asset_file_ext = 'pdf'
SQL
    return 1;
}

sub param_edit_field {
    my ($cb, $app, $param, $tmpl) = @_;

    my $plugin = $app->component('ImageUploadUtility')
        or croak 'Plugin not found.'; 
    return 1 unless $plugin->enable_pdf_thumbnail;
    
    my $blog_id = $app->param('blog_id') || 0;

    foreach (@{$param->{customfield_types_loop}}) {
        if($_->{key} eq 'pdf') {
            $_->{show_column} = 0;
        }
    }

    return 1;
}

sub filtered_list_param_asset {
    my ($cb, $app, $res, $objs) = @_;
    my $plugin = $app->component('ImageUploadUtility')
        or croak 'Plugin not found.'; 
    return 1 unless $plugin->enable_pdf_thumbnail;
    $res->{objects}
        or return 1;
    return unless $objs && ref $objs eq 'ARRAY';
    my $count = scalar @$objs;
    for(my $i=0; $i<$count; $i++) {
        my $asset = $objs->[$i];
        if($asset->class_type eq 'pdf') {
            $res->{objects}->[$i]->[1] =~ s!file-type pdf picture!file-type file picture!;
            $res->{objects}->[$i]->[1] =~ s!asset-type-pdf!asset-type-file!;
            $res->{objects}->[$i]->[1] =~ s!pdf-45.png!file-45.png!;
        }
    }

    return 1;
}

sub multiupload_script {
    my ($plugin, $app, $tmpl_ref) = @_;

    my $script = <<SCRIPT;
<mt:setvarblock name="js_include" append="1">
    <script type="text/javascript">
        MultiUploadSettings = <mt:if name="iuu_multiupload_settings"><mt:var name="iuu_multiupload_settings" /><mt:else>{}</mt:if>;
    </script>
    <script src="<mt:var name="static_uri" />plugins/ImageUploadUtility/js/ajaxq.js?v=<mt:var name="mt_version_id" escape="url" />"></script>
<mt:unless name="optimize_ui">
    <script data-main="<mt:var name="static_uri" />plugins/ImageUploadUtility/js/multiupload.js" src="<mt:var name="static_uri" />plugins/ImageUploadUtility/js/require.js?v=<mt:var name="mt_version_id" escape="url" />"></script>
</mt:unless>
</mt:setvarblock>
<mt:setvarblock name="jq_js_include" append="1">
<mt:if name="optimize_ui">
    var src = "<mt:var name="static_uri" />plugins/ImageUploadUtility/js/multiupload.min.js?v=<mt:var name="mt_version_id" escape="url" />";
    jQuery('<script>').attr({
        src: src,
        type: 'text/javascript',
    }).appendTo('head');
</mt:if>
</mt:setvarblock>
SCRIPT

    #my $target = quotemeta(<<TMPL);
    my $target = <<TMPL;
<mt:include name="layout/default.tmpl"[^>]*>
TMPL

    $$tmpl_ref =~ s!($target)!$script$1!;

    return 1;
}

sub multiupload_params {
    my ($plugin, $app, $param, $tmpl) = @_;
    # $app->log(sprintf("edit_param: %s", Data::Dumper->Dump([ $param ])));

    # plugin settings
    require MT::Util;
    my $blog_id = $app->param('blog_id') || 0;
    my $config_hash = $plugin->get_config_hash('blog:'.$blog_id);
    my $params = {}; 
    foreach my $key (keys %$config_hash) {
        if($key =~ m!^image_upload_utility_(.+)!) {
            my $new_key = $1;
            my $value = $config_hash->{$key};
            if($new_key eq 'rename_format') {
                $value = MT::Util::encode_html($value);
            }
            $params->{$new_key} = $value;
        }
    }
    $params->{blog_id} = $blog_id;
    $params->{mt_uri} = $app->mt_uri;
    $params->{magic_token} = $app->current_magic;

    # image_defaults
    my $blog = $app->blog
        or return 1;
    $params->{can_save_image_defaults} 
        = $app->{perms}->can_do('save_image_defaults') ? 1 : 0;
    $params->{popup} = $blog->image_default_popup     ? 1 : 0;
    # $params->{wrap_text}  = $blog->image_default_wrap_text ? 1 : 0;
    $params->{make_thumb} = $blog->image_default_thumb     ? 1 : 0;
    $params->{ 'align_' . $_ }
        = ( $blog->image_default_align || 'none' ) eq $_ ? 1 : 0
        for qw(none left center right);
    $params->{thumb_width} = $blog->image_default_width || 0;

    # custom fields for drag and drop
    my @custom_fields;
    foreach my $field (@{$param->{field_loop}}) {
        if(defined $field->{'type'} && $field->{'type'} eq 'image') {
            push @custom_fields, $field->{field_id}; 
        }
    }
    if(@custom_fields) {
        $params->{custom_fields} = \@custom_fields;
    }

    # content fields for drag and drop
    my @content_fields;
    foreach my $cf (@{$param->{fields}}) {
        if($cf->{type} eq 'multi_line_text') {
            push @content_fields, 'contentblock'.$cf->{content_field_id};
        }
        elsif($cf->{type} eq 'asset_image' && $cf->{options}->{allow_upload} == 1) {
            # options min, max, multiple, allow_upload
            push @content_fields, 'asset-field-'.$cf->{content_field_id};
        }
    }
    if(@content_fields) {
        $params->{content_fields} = \@content_fields;
    }

    if($app->config->DebugMode) {
        $params->{debug_mode} = 1;
    }

    my $perms = $app->permissions;
    if($perms && $perms->can_do('upload')) {
        # startup params
        require MT::CMS::Asset;
        my $upload_params = MT::CMS::Asset::_set_start_upload_params($app, $params);
        # print STDERR "upload_params: " . Data::Dumper->Dump([ $upload_params ]);
    }
    else {
        $params->{permission_denied} = 1;
        delete $params->{custom_fields};
        delete $params->{content_fields};
    }
    
    # upload compat mode
    $params->{upload_compat} = $plugin->is_upload_compat ? 1 : 0;

    # admin theme id
    $params->{admin_theme_id} = $app->config('AdminThemeId') || '';

    my $settings = MT::Util::to_json($params, { pretty => 1 });
    $param->{iuu_multiupload_settings} = $settings;
    $app->log('multiupload_params: ' . $settings) if DEBUG;
    return 1;
}

sub param_asset_insert {
    my ($cb, $app, $param, $tmpl) = @_;
        
    my $nodes = $tmpl->getElementsByName('upload_html');
    foreach my $node (@$nodes) {
        if($node->tag eq 'if') {
            my $innerHTML = $node->innerHTML();
            $innerHTML .= <<SCRIPT;
window.parent.jQuery('#<mt:var name="edit_field" escape="js">').trigger('change');
SCRIPT
            $node->innerHTML($innerHTML);
        }
    }

    return 1;
}

sub source_async_asset_upload {
    my ($cb, $app, $tmpl_ref, $file) = @_;
    my $pattern = quotemeta(<<'HTML');
</form>
HTML
    my $replacement = <<'HTML';
<!-- Async Asset Upload -->
<mt:unless name="image_upload_utility_watermark_always_use">
<__trans_section component="ImageUploadUtility">
<mtapp:setting
    id="image_upload_utility_watermark_insertion"
    label_class="left-label"
    label="<__trans phrase="Watermark" />">
    <input type="checkbox" id="image_upload_utility_watermark_insertion" name="image_upload_utility_watermark_insertion" value="1" />
    <label for="image_upload_utility_watermark_insertion"><__trans phrase="Insert wartermark" /></label>
</mtapp:setting>
</__trans_section>
</mt:unless>
HTML
    $$tmpl_ref =~ s!($pattern)!$replacement$1!;

    my $js_pattern = qr/ +return fd;/;
    my $js_replacement = <<'SCRIPT';
  var insertWatermark = 0;
  if ( jQuery('#image_upload_utility_watermark_insertion').prop('checked') ) {
    insertWatermark = 1;
  }
  fd.append('image_upload_utility_watermark_insertion', insertWatermark);

SCRIPT
    $$tmpl_ref =~ s!($js_pattern)!$js_replacement$1!;

    $app->log('source_async_asset_upload: ') if DEBUG;
    return 1;
}

sub param_async_asset_upload {
    my ($cb, $app, $param, $tmpl) = @_;
    return 1;
}

sub param_asset_modal {
    my ($cb, $app, $param, $tmpl) = @_;
    my $plugin = $app->component('ImageUploadUtility')
        or croak 'Plugin not found.'; 
    my $blog_id = $app->param('blog_id') || 0;
    $param->{image_upload_utility_watermark_always_use} = $plugin->watermark_always_use($blog_id);
    # $app->log('param_asset_modal: ' . Data::Dumper->Dump([ $param ])) if DEBUG;
    return 1;
}

sub replace_protected_pdf_icon {
    my $result = shift;
    if ($result
        && exists $result->{asset}
        && exists $result->{asset}->{thumbnail}
        && $result->{asset}->{thumbnail} =~ m!pdf-45.png$!) {
        $result->{asset}->{thumbnail} =~ s!pdf-45!file-45!;
    }
    return $result;
}

sub source_edit_asset {
    my ($cb, $app, $tmpl_ref, $file) = @_;
    my $pattern = quotemeta(<<'TMPL');
        <img src="<mt:var name="static_uri">images/asset/<mt:var name="asset_type">-45.png" width="45" height="45" />
TMPL
    my $replacement = <<'TMPL';
        <img src="<mt:var name="static_uri">images/asset/<mt:if name="asset_type" eq="pdf">file<mt:else><mt:var name="asset_type"></mt:if>-45.png" width="45" height="45" />
TMPL
    $$tmpl_ref =~ s!$pattern!$replacement!;

    return 1;
}

sub is_pdf_supported {
    my $plugin = MT->instance()->component('ImageUploadUtility')
        or return;
    require File::Spec;
    my $path = File::Spec->catfile($plugin->path, 'data', 'check.pdf');
    my $graphic_drv = '';
    eval {
        $graphic_drv = $plugin->get_image_driver;
    };
    return if $@;
    my $m = $graphic_drv->new
        or return;
    my @info = $m->QueryFormat('pdf');
    # Index 3 indicates whether the decorder is available or not.
    return (scalar @info && $info[3]);
}

sub param_asset_upload {
    my ($cb, $app, $param, $tmpl) = @_;
    my $plugin = $app->component('ImageUploadUtility')
        or croak 'Plugin not found.'; 
    my $blog_id = $app->param('blog_id') || 0;
    $param->{image_upload_utility_watermark_always_use} = $plugin->watermark_always_use($blog_id);
    $app->log('param_asset_upload: ' . Data::Dumper->Dump([ $param ])) if DEBUG;
    return 1;
}

sub cms_upload_file {
    my ($cb, %args) = @_;
    my $app = MT->instance;
    my $asset_map = $app->request('iuu_asset_map')
        or return 1;
    my $type = $args{type} || '';
    return 1 unless $type eq 'thumbnail' || $type eq 'popup';
    my $asset = $args{asset};
    my $parent_id = $asset->parent
        or return 1;
    $asset_map->{$parent_id}->{$type} = $asset->id;
    $app->request('iuu_asset_map', $asset_map);
    return 1;
}

1;
__END__


