# ImageUploadUtility - (c) 2012-2017 Juxtaposition. All Rights Reserved.
# This code cannot be redistributed without permission from juxtaposition.jp
# For more information, consult your ImageUploadUtility license.
#

package MT::Asset::PDF;

use strict;
use warnings;
use utf8;
use base qw( MT::Asset::Image );
use MT::Blog;
use MT::Website;
use File::Spec;
use Carp qw( longmess );
use Capture::Tiny qw( capture );

use constant DEBUG => 0;

__PACKAGE__->install_properties(
    {
        class_type => 'pdf',
        column_defs => {
            'image_width'  => 'integer meta',
            'image_height' => 'integer meta',
            'protected' => 'integer meta',
        },
        child_of => [ 'MT::Blog', 'MT::Website', ],
    }
);

sub extensions {
    my $pkg = shift;
    # return $pkg->SUPER::extensions([ qr/pdf/i ]);
    return [ qr/pdf/i ];
}

sub class_label {
    MT->component('ImageUploadUtility')->translate('Image(PDF)');
}

sub class_label_plural {
    MT->component('ImageUploadUtility')->translate('Images(PDF)');
}

sub image_height {
    my $asset = shift;
    my $height = $asset->meta('image_height', @_);
    return $height if $height || @_;

    if($asset->meta('protected')) {
        return;
    }

    if(!-e $asset->file_path || !-r $asset->file_path) {
        return;
    }

    if(!$asset->{__pdf_image}) {
        # NOTE: __pdf_imageはcreate_pdf_thumbnail_image内でセットされる  
        my $image = $asset->create_pdf_thumbnail_image
            or return;
    }

    my $h = $asset->{__pdf_image}->Get('Height');
    $asset->meta('image_height', $h);
    return $h;
}

sub image_width {
    my $asset = shift;
    my $width = $asset->meta('image_width', @_);
    return $width if $width || @_;

    if($asset->meta('protected')) {
        return;
    }

    if(!-e $asset->file_path || !-r $asset->file_path) {
        return;
    }

    if(!$asset->{__pdf_image}) {
        # NOTE: __pdf_imageはcreate_pdf_thumbnail_image内でセットされる  
        my $image = $asset->create_pdf_thumbnail_image
            or return;
    }

    my $w = $asset->{__pdf_image}->Get('Width');
    $asset->meta('image_width', $w);
    return $w;
}

sub has_thumbnail {
    my $asset = shift;
    return 0 unless MT->component('ImageUploadUtility')->enable_pdf_thumbnail;
    return $asset->is_protected ? 0 
        : $asset->image_width ? 1 : 0;
}

sub is_protected {
    my $asset = shift;
    return $asset->meta('protected') ? 1 : 0;
}

sub insert_options {
    my $asset = shift;
    my ($param) = @_;
    if($asset->is_protected) {
        return undef;
    }
    return $asset->SUPER::insert_options(@_);
}

sub on_upload {
    my $asset = shift;
    my ($param) = @_;
    if($asset->is_protected) {
        return 1;
    }
    MT->instance->log('on_upload: ' . Data::Dumper->Dump([ $param ])) if DEBUG;
    my $result;
    {
        local $param->{popup} = 0;
        $result = $asset->SUPER::on_upload(@_);

    }
    return $result;
}

sub stock_icon_url {
    my $asset = shift;
    my (%param) = @_;
    my $size = $param{Width} || $param{Height} || 45;
    unless($size == 45 || $size == 20) {
        $size = 45;
    }
    my $thumb_url = MT->static_path . 'images/asset/file-' . $size . '.png';
    return $thumb_url;
}

sub thumbnail_filename {
    my $asset   = shift;
    my (%param) = @_;
    return if $asset->is_protected;
    my $file    = $asset->file_name or return;

    require MT::Util;
    my $format = $param{Format} || MT->translate('%f-thumb-%wx%h-%i%x');
    my $width  = $param{Width}  || 'auto';
    my $height = $param{Height} || 'auto';
    $file =~ s/\.\w+$//;
    my $base = File::Basename::basename($file);
    my $id   = $asset->id;
    my $ext  = lc( $param{Type} || '' ) || 'png'; # default 'png'
    $ext = '.' . $ext;
    $format =~ s/%w/$width/g;
    $format =~ s/%h/$height/g;
    $format =~ s/%f/$base/g;
    $format =~ s/%i/$id/g;
    $format =~ s/%x/$ext/g;
    return $format;
}

sub thumbnail_file {
    my $asset = shift;
    my (%param) = @_;

    return if $asset->is_protected;
    my $blog = $param{Blog} || $asset->blog;

    require MT::FileMgr;
    my $fmgr = $blog ? $blog->file_mgr : MT::FileMgr->new('Local');
    return unless $fmgr;

    my $file_path = $asset->file_path;
    return unless $fmgr->file_size($file_path);

    my $asset_cache_path = $asset->_make_cache_path($param{Path});
    
    my ($i_w, $i_h) = ($asset->image_width, $asset->image_height); 
    return unless $i_h && $i_w;

    my $auto_size = 1;
    if($param{Square}) {
        require MT::Image;
        my %square = MT::Image->inscribe_square(Width => $i_w, Height => $i_h);
        ($i_h, $i_w) = @square{qw( Size Size )};
        if($param{Width} && !$param{Height}) {
            $param{Height} = $param{Width};
        }
        elsif(!$param{Width} && $param{Height}) {
            $param{Width} = $param{Height};
        }
        $auto_size = 0;
    }
    if(my $scale = $param{Scale}) {
        $param{Width} = int(($i_w * $scale) / 100);
        $param{Height} = int(($i_h * $scale) / 100);
        $auto_size = 0;
    }
    if(!exists $param{Width} && !exists $param{Height}) {
        $param{Width} = $i_w;
        $param{Height} = $i_h;
        $auto_size = 0;
    }

    my ($n_h, $n_w, $scaled) = MT::Asset::Image::_get_dimension($i_h, $i_w, $param{Height}, $param{Width});
    if($auto_size && defined $scaled && $scaled eq 'h') {
        delete $param{Width} if exists $param{Width};
    }
    elsif($auto_size && defined $scaled && $scaled eq 'w') {
        delete $param{Height} if exists $param{Height};
    }

    my $file = $asset->thumbnail_filename(%param) or return;
    my $thumbnail = File::Spec->catfile($asset_cache_path, $file);

    if($fmgr->exists($thumbnail)
        && ($fmgr->file_mod_time($thumbnail) >= $fmgr->file_mod_time($file_path))) {
        my $already_exists = 1;
        if($asset->image_width != $asset->image_height) {
            open(my $fh, "<", $thumbnail)
                or return $asset->error("Error open file: ".$!);
            require MT::Image;
            my ($t_w, $t_h) = MT::Image->get_image_info(Fh => $fh);
            close $fh;

            if(($param{Square} && $t_h != $t_w)
                || (!$param{Square} && $t_h == $t_w)) {
                $already_exists = 0;
            }
        }
        return ($thumbnail, $n_w, $n_h) if $already_exists;
    }

    return undef unless $fmgr->can_write($asset_cache_path);

    my $data;

    my $pdf_image = $asset->create_pdf_thumbnail_image
        or return;
    my $blob = $pdf_image->ImageToBlob
        or return;

    require MT::Image;
    my $img = MT::Image->new(Data => $blob, Type => 'png')
        or return $asset->error(MT::Image->errstr);

    if($param{Square}) {
        ($data) = $img->make_square
            or return $asset->error(
            MT->translate("Error cropping image: [_1]", $img->errstr));
    }

    ($data) = $img->scale(Height => $n_h, Width => $n_w)
        or return $asset->error(
        MT->translate("Error scaling image: [_1]", $img->errstr));

    if(my $type = $param{Type}) {
        ($data) = $img->convert(Type => $type)
            or return $asset->error(
            MT->translate("Error converting image: [_1]", $img->errstr)
        );
    }
    $fmgr->put_data($data, $thumbnail, 'upload')
        or return $asset->error(
            MT->translate("Error creating thumbnail file: [_1]", $fmgr->errstr)
        );

    return ($thumbnail, $n_w, $n_h);
}

sub create_pdf_thumbnail_image {
    my ($asset) = @_;

    my $plugin = MT->component('ImageUploadUtility');
    $plugin->enable_pdf_thumbnail
        or return;

    if($asset->{__pdf_image}) {
        return $asset->{__pdf_image};
    }

    my $app = MT->instance;
    my $graphic_drv = $plugin->get_image_driver;
    my $img = $graphic_drv->new;
    unless($img) {
        $app->log("failed to create $graphic_drv instance");
        die "failed to create $graphic_drv instance";
    }

    require File::Spec;
    # Redirect STDOUT & STDERR because PDF read error is written to there.
    my $read_err;
    my ($stdout, $stderr) = capture {
        $read_err = $img->Read( $asset->file_path . '[0]' );
    };

    if("$read_err" || $stderr) {
        $asset->meta('protected', 1);
        $app->log(
            $plugin->translate("Failed to create PDF thumbnail: [_1]: [_2]", $asset->file_name, $read_err . ', ' . $stderr)
        );
        return;
    }

    $img = $img->Flatten();
    $img->Set( magick => 'png' );
    $asset->{__pdf_image} = $img;
    return $img;
}

sub as_html {
    my $asset = shift;
    my ($param) = @_;
    my $text;
    {
        local $param->{popup} = 0;
        $text = $asset->SUPER::as_html(@_);

    }
    return $text unless $text && $param->{popup};

    MT->instance->log('as_html: ' . Data::Dumper->Dump([ $param ])) if DEBUG;

    $text =~ s{(<a href="[^"]+")(>.+?</a>)}{$1 target="_blank"$2};
    return $text;
}

sub is_metadata_broken {
    return 0;
}

1;
__END__


